"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = builder;
exports.configure = exports.Col = exports.Row = exports.Condition = exports.Focus = exports.Tabs = exports.Default = exports.Block = exports.Collapse = exports.Layout = void 0;

var _models = require("./models");

var _layout = require("./layout");

var _visitorManager = _interopRequireDefault(require("./visitorManager"));

var _validator = _interopRequireDefault(require("./validator"));

var _configure = _interopRequireDefault(require("./configure"));

var _utils = require("./utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Date.prototype.toString.call(Reflect.construct(Date, [], function () {})); return true; } catch (e) { return false; } }

function _construct(Parent, args, Class) { if (isNativeReflectConstruct()) { _construct = Reflect.construct; } else { _construct = function _construct(Parent, args, Class) { var a = [null]; a.push.apply(a, args); var Constructor = Function.bind.apply(Parent, a); var instance = new Constructor(); if (Class) _setPrototypeOf(instance, Class.prototype); return instance; }; } return _construct.apply(null, arguments); }

function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

// layout
var Layout = 'Layout';
exports.Layout = Layout;

var Collapse = function Collapse(_ref) {
  var attributes = _ref.attributes,
      children = _ref.children;
  return builder(Layout, _extends({
    ui: "collapse"
  }, attributes), children);
};

exports.Collapse = Collapse;

var Block = function Block(_ref2) {
  var attributes = _ref2.attributes,
      children = _ref2.children;
  return builder(Layout, _extends({
    ui: "block"
  }, attributes), children);
};

exports.Block = Block;

var Default = function Default(_ref3) {
  var attributes = _ref3.attributes,
      children = _ref3.children;
  return builder(Layout, _extends({
    ui: "default"
  }, attributes), children);
};

exports.Default = Default;

var Tabs = function Tabs(_ref4) {
  var attributes = _ref4.attributes,
      children = _ref4.children;
  return builder(Layout, _extends({
    ui: "tabs"
  }, attributes), children);
};

exports.Tabs = Tabs;

var Focus = function Focus(_ref5) {
  var attributes = _ref5.attributes,
      children = _ref5.children;
  return builder(Layout, _extends({
    ui: "focus"
  }, attributes), children);
};

exports.Focus = Focus;

var Condition = function Condition(_ref6) {
  var attributes = _ref6.attributes,
      children = _ref6.children;
  return builder(Layout, _extends({
    ui: "condition"
  }, attributes), children);
};

exports.Condition = Condition;

var Row = function Row(_ref7) {
  var attributes = _ref7.attributes,
      children = _ref7.children;
  return builder(Layout, _extends({
    ui: "row"
  }, attributes), children);
};

exports.Row = Row;

var Col = function Col(_ref8) {
  var attributes = _ref8.attributes,
      children = _ref8.children;
  return builder(Layout, _extends({
    ui: "col"
  }, attributes), children);
}; // validator config


exports.Col = Col;
var configure = _configure.default.configure; // jsx pragma

exports.configure = configure;

function builder(tag, attributes) {
  for (var _len = arguments.length, children = new Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
    children[_key - 2] = arguments[_key];
  }

  // flattern children
  children = children.reduce(function (result, child) {
    return result.concat(child);
  }, []).filter(function (child) {
    return child;
  });
  children = (attributes || {}).children || children;

  if (typeof tag === 'function') {
    return tag({
      attributes: attributes,
      children: children
    });
  }

  attributes = attributes || {}; // transform some value to react intl component

  i18n(attributes);

  switch (tag) {
    case 'string':
      return createJSON(_models.StringModel, [attributes, children]);

    case 'number':
      return createJSON(_models.NumberModel, [attributes, children]);

    case 'boolean':
      return createJSON(_models.BooleanModel, [attributes, children]);

    case 'array':
      return createJSON(_models.ArrayModel, [attributes, children]);

    case 'object':
      return createJSON(_models.ObjectModel, [attributes, children]);

    case 'arrayType':
      return createJSON(_models.ArrayModel, [_objectSpread({}, attributes, {
        defOnly: true
      }), children]);

    case 'objectType':
      return createJSON(_models.ObjectModel, [_objectSpread({}, attributes, {
        defOnly: true
      }), children]);

    case 'geoPoint':
      return createJSON(_models.GeoPointModel, [attributes, children]);

    case 'dateTime':
      return createJSON(_models.DateModel, [attributes, children]);

    case 'file':
      return createJSON(_models.FileModel, [attributes, children]);

    case 'image':
      return createJSON(_models.ImageModel, [attributes, children]);

    case 'relation':
      return createJSON(_models.RelationModel, [attributes, children]);

    case 'json':
      return createJSON(_models.JsonModel, [attributes, children]);

    case 'component':
      return createJSON(_models.ComponentModel, [attributes, children]);

    case 'enum':
      return new createJSON(_models.EnumModel, [attributes, children]);

    case 'page':
      return new _models.PageModel(attributes, children).toJson();

    case 'root':
      return new _models.RootModel(attributes, children).toJson();

    case 'toolbar':
      {
        return children.reduce(function (result, child) {
          result[child.type] = _objectSpread({}, child);
          return result;
        }, _objectSpread({}, attributes, {
          __TOOLBAR__: true
        }));
      }

    case 'sorter':
      {
        return _objectSpread({}, attributes, {
          type: 'sorter'
        });
      }

    case 'pagination':
      {
        return _objectSpread({}, attributes, {
          type: 'pagination'
        });
      }

    case 'textFilter':
      {
        return _objectSpread({}, attributes, {
          type: 'text'
        });
      }

    case 'numberFilter':
      {
        return _objectSpread({}, attributes, {
          type: 'number'
        });
      }

    case 'dateFilter':
      {
        return _objectSpread({}, attributes, {
          type: 'date'
        });
      }

    case 'relationSelectFilter':
      {
        return _objectSpread({}, attributes, {
          type: 'relation'
        });
      }

    case 'selectFilter':
      {
        return _objectSpread({}, attributes, {
          type: 'select'
        });
      }

    case 'filter':
      {
        return _objectSpread({}, attributes, {
          type: 'filter',
          filters: children
        });
      }

    case 'actions':
      {
        return _objectSpread({}, attributes, children.reduce(function (result, child) {
          result[child.type] = child;
          return result;
        }, {}), {
          type: 'actions'
        });
      }

    case 'export':
      {
        return _objectSpread({}, attributes, {
          type: 'export'
        });
      }

    case 'import':
      {
        return _objectSpread({}, attributes, {
          type: 'import'
        });
      }

    case 'Layout':
      {
        var _createLayoutVisitor = (0, _layout.createLayoutVisitor)(attributes, children),
            visitor = _createLayoutVisitor.visitor,
            cannerKey = _createLayoutVisitor.cannerKey;

        var injectLayout;

        if (attributes.injectValue) {
          injectLayout = (0, _layout.createLayoutVisitor)(_objectSpread({}, attributes, {
            layoutType: 'injection'
          }), children);
        }

        _visitorManager.default.addVisitor(visitor);

        if (injectLayout) {
          _visitorManager.default.addVisitor(injectLayout.visitor);
        }

        return children.map(function (child) {
          if (child[_layout.CANNER_KEY]) {
            child[_layout.CANNER_KEY].push(cannerKey);
          } else {
            child[_layout.CANNER_KEY] = [cannerKey];
          }

          if (injectLayout) {
            child[_layout.CANNER_KEY].push(injectLayout.cannerKey);
          }

          return child;
        });
      }

    default:
      throw new Error("unsupported type '".concat(tag, "'"));
  }
}

function createJSON(Model, args) {
  var json = _construct(Model, _toConsumableArray(args)).toJson();

  _validator.default.validate(json);

  return json;
}

function i18n(attrs) {
  ['title', 'description', 'uiParams', 'label', 'options', 'placeholder'].forEach(function (key) {
    if (key in attrs) {
      attrs[key] = (0, _utils.getIntlMessage)(attrs[key]);
    }
  });
}