"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _utils = require("./utils");

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var TYPES = ['string', 'number', 'boolean', 'array', 'object', 'relation', 'json', 'component'];
var CANNER_DATA_TYPES = Object.keys(_utils.componentMap._map);

var Validator = function Validator() {
  var _this = this;

  _classCallCheck(this, Validator);

  _defineProperty(this, "throwError", true);

  _defineProperty(this, "setConfig", function (config) {
    if ('throwError' in config) {
      _this.throwError = config.throwError;
    }
  });

  _defineProperty(this, "validatePackageName", function (packageName) {
    var type = _this.parseType(packageName);

    if (TYPES.indexOf(type) === -1) {
      var message = "Invalid package name '".concat(packageName, "', type should be one of ").concat(JSON.stringify(TYPES));
      return _this.invalid(message);
    }

    return true;
  });

  _defineProperty(this, "validateCannerDataType", function (cannerDataType) {
    if (CANNER_DATA_TYPES.indexOf(cannerDataType) === -1) {
      var message = "Invalid canner data type '".concat(cannerDataType, ", should be one of ").concat(JSON.stringify(CANNER_DATA_TYPES), "'");
      return _this.invalid(message);
    }

    return true;
  });

  _defineProperty(this, "validate", function (json) {
    var packageName = json.packageName,
        type = json.type;

    if (type === 'component') {
      return true;
    }

    if (packageName === '@canner/antd-object-fieldset') {
      return true;
    }

    return _this.validatePackageName(packageName) && _this.validateCannerDataType(type);
  });

  _defineProperty(this, "invalid", function () {
    var message = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';

    if (_this.throwError) {
      throw new Error(message);
    } else {
      // eslint-disable-next-line
      console.error(message);
      return false;
    }
  });

  _defineProperty(this, "parseType", function (packageName) {
    return packageName.split('/').slice(-1)[0].split('-')[1];
  });
};

var _default = new Validator();

exports.default = _default;